@@ -1,10 +1,10 @@
 ###########################################################################
 # Module for ABI Compliance Checker to compare Operating Systems
 #
-# Copyright (C) 2009-2010 The Linux Foundation
 # Copyright (C) 2009-2011 Institute for System Programming, RAS
 # Copyright (C) 2011-2012 Nokia Corporation and/or its subsidiary(-ies)
-# Copyright (C) 2011-2013 ROSA Laboratory
+# Copyright (C) 2011-2012 ROSA Laboratory
+# Copyright (C) 2012-2015 Andrey Ponomarenko's ABI Laboratory
 #
 # Written by Andrey Ponomarenko
 #
@@ -26,9 +26,9 @@ use File::Temp qw(tempdir);
 use Cwd qw(abs_path cwd);
 use Fcntl;
 
-my ($Debug, $Quiet, $LogMode, $CheckHeadersOnly, $SystemRoot, $MODULES_DIR, $GCC_PATH,
-$CrossPrefix, $TargetSysInfo, $TargetLibraryName, $CrossGcc, $UseStaticLibs, $NoStdInc,
-$OStarget, $BinaryOnly, $SourceOnly);
+my ($Debug, $Quiet, $LogMode, $CheckHeadersOnly, $SystemRoot, $GCC_PATH,
+$CrossPrefix, $TargetSysInfo, $TargetLibraryName, $CrossGcc, $UseStaticLibs,
+$NoStdInc, $OStarget, $BinaryOnly, $SourceOnly);
 
 my $OSgroup = get_OSgroup();
 my $TMP_DIR = tempdir(CLEANUP=>1);
@@ -987,7 +987,9 @@ sub cmpSystems($$$)
     my $Styles = readModule("Styles", "CmpSystems.css");
     
     $SYS_REPORT = composeHTML_Head($Title, $Keywords, $Description, $Styles, "")."\n<body>\n<div>".$SYS_REPORT."</div>\n";
-    $SYS_REPORT .= "<br/><br/><br/><hr/>\n".getReportFooter($SystemName2, 1)."<div style='height:999px;'></div>\n</body></html>";
+    $SYS_REPORT .= "<br/><br/><br/>\n";
+    $SYS_REPORT .= getReportFooter();
+    $SYS_REPORT .= "</body></html>\n";
     
     if($SourceOnly) {
         $SYS_REPORT = "<!-\- kind:source;".join(";", @{$META_DATA{"Source"}})." -\->\n".$SYS_REPORT;
@@ -1062,7 +1064,8 @@ sub readSystemDescriptor($)
             exitStatus("Access_Error", "can't access \'$Path\'");
         }
         $Path = get_abs_path($Path);
-        $SysDescriptor{"Libs"}{clean_path($Path)} = 1;
+        $Path=~s/[\/\\]+\Z//g;
+        $SysDescriptor{"Libs"}{$Path} = 1;
     }
     foreach my $Path (split(/\s*\n\s*/, parseTag(\$Content, "search_libs")))
     { # target libs
@@ -1070,7 +1073,8 @@ sub readSystemDescriptor($)
             exitStatus("Access_Error", "can't access directory \'$Path\'");
         }
         $Path = get_abs_path($Path);
-        $SysDescriptor{"SearchLibs"}{clean_path($Path)} = 1;
+        $Path=~s/[\/\\]+\Z//g;
+        $SysDescriptor{"SearchLibs"}{$Path} = 1;
     }
     foreach my $Path (split(/\s*\n\s*/, parseTag(\$Content, "skip_libs")))
     { # skip libs
@@ -1082,7 +1086,8 @@ sub readSystemDescriptor($)
             exitStatus("Access_Error", "can't access \'$Path\'");
         }
         $Path = get_abs_path($Path);
-        $SysDescriptor{"Headers"}{clean_path($Path)} = 1;
+        $Path=~s/[\/\\]+\Z//g;
+        $SysDescriptor{"Headers"}{$Path} = 1;
     }
     foreach my $Path (split(/\s*\n\s*/, parseTag(\$Content, "search_headers")))
     {
@@ -1090,7 +1095,8 @@ sub readSystemDescriptor($)
             exitStatus("Access_Error", "can't access directory \'$Path\'");
         }
         $Path = get_abs_path($Path);
-        $SysDescriptor{"SearchHeaders"}{clean_path($Path)} = 1;
+        $Path=~s/[\/\\]+\Z//g;
+        $SysDescriptor{"SearchHeaders"}{$Path} = 1;
     }
     foreach my $Path (split(/\s*\n\s*/, parseTag(\$Content, "tools")))
     {
@@ -1098,12 +1104,14 @@ sub readSystemDescriptor($)
             exitStatus("Access_Error", "can't access directory \'$Path\'");
         }
         $Path = get_abs_path($Path);
-        $Path = clean_path($Path);
+        $Path=~s/[\/\\]+\Z//g;
         $SysDescriptor{"Tools"}{$Path} = 1;
         push(@Tools, $Path);
     }
-    foreach my $Path (split(/\s*\n\s*/, parseTag(\$Content, "gcc_options"))) {
-        $SysDescriptor{"GccOpts"}{clean_path($Path)} = 1;
+    foreach my $Path (split(/\s*\n\s*/, parseTag(\$Content, "gcc_options")))
+    {
+        $Path=~s/[\/\\]+\Z//g;
+        $SysDescriptor{"GccOpts"}{$Path} = 1;
     }
     if($SysDescriptor{"CrossPrefix"} = parseTag(\$Content, "cross_prefix"))
     { # <cross_prefix> section of XML descriptor
@@ -1135,7 +1143,6 @@ sub initModule($)
     $CheckHeadersOnly = $S->{"CheckHeadersOnly"};
     
     $SystemRoot = $S->{"SystemRoot"};
-    $MODULES_DIR = $S->{"MODULES_DIR"};
     $GCC_PATH = $S->{"GCC_PATH"};
     $TargetSysInfo = $S->{"TargetSysInfo"};
     $CrossPrefix = $S->{"CrossPrefix"};
@@ -1197,7 +1204,7 @@ sub filter_format($)
     }
 }
 
-sub read_sys_descriptor($)
+sub readSysDescriptor($)
 {
     my $Path = $_[0];
     my $Content = readFile($Path);
@@ -1256,36 +1263,31 @@ sub read_sys_descriptor($)
     return \%DInfo;
 }
 
-sub read_sys_info($)
+sub readSysInfo($)
 {
     my $Target = $_[0];
-    my $SYS_INFO_PATH = $MODULES_DIR."/Targets";
-    if(-d $SYS_INFO_PATH."/".$Target)
-    { # symbian, windows
-        $SYS_INFO_PATH .= "/".$Target;
+    
+    if(not $TargetSysInfo) {
+        exitStatus("Error", "system info path is not specified");
     }
-    else
-    { # default
-        $SYS_INFO_PATH .= "/unix";
-    }
-    if($TargetSysInfo)
-    { # user-defined target
-        $SYS_INFO_PATH = $TargetSysInfo;
-    }
-    if(not -d $SYS_INFO_PATH) {
-        exitStatus("Module_Error", "can't access \'$SYS_INFO_PATH\'");
+    if(not -d $TargetSysInfo) {
+        exitStatus("Module_Error", "can't access \'$TargetSysInfo\'");
     }
     # Library Specific Info
     my %SysInfo = ();
-    if(not -d $SYS_INFO_PATH."/descriptors/") {
-        exitStatus("Module_Error", "can't access \'$SYS_INFO_PATH/descriptors\'");
-    }
-    foreach my $DPath (cmd_find($SYS_INFO_PATH."/descriptors/","f","",1))
+    if(-d $TargetSysInfo."/descriptors/")
     {
-        my $LSName = get_filename($DPath);
-        $LSName=~s/\.xml\Z//;
-        $SysInfo{$LSName} = read_sys_descriptor($DPath);
+        foreach my $DPath (cmd_find($TargetSysInfo."/descriptors/","f","",1))
+        {
+            my $LSName = get_filename($DPath);
+            $LSName=~s/\.xml\Z//;
+            $SysInfo{$LSName} = readSysDescriptor($DPath);
+        }
     }
+    else {
+        printMsg("WARNING", "can't find \'$TargetSysInfo/descriptors\'");
+    }
+    
     # Exceptions
     if(check_gcc($GCC_PATH, "4.4"))
     { # exception for libstdc++
@@ -1299,11 +1301,16 @@ sub read_sys_info($)
     { # GL/gl.h: No such file
         $SysInfo{"libSDL"}{"skip_headers"}=["SDL_opengl.h"];
     }
+    
     # Common Info
-    if(not -f $SYS_INFO_PATH."/common.xml") {
-        exitStatus("Module_Error", "can't access \'$SYS_INFO_PATH/common.xml\'");
+    my $SysCInfo = {};
+    if(-f $TargetSysInfo."/common.xml") {
+        $SysCInfo = readSysDescriptor($TargetSysInfo."/common.xml");
     }
-    my $SysCInfo = read_sys_descriptor($SYS_INFO_PATH."/common.xml");
+    else {
+        printMsg("Module_Error", "can't find \'$TargetSysInfo/common.xml\'");
+    }
+    
     my @CompilerOpts = ();
     if($SysDescriptor{"Name"}=~/maemo|meego/i) {
         push(@CompilerOpts, "-DMAEMO_CHANGES", "-DM_APPLICATION_NAME=\\\"app\\\"");
@@ -1418,7 +1425,7 @@ sub dumpSystem($)
         "libc",
         "libpthread"
     );
-    my ($SysInfo, $SysCInfo) = read_sys_info($OStarget);
+    my ($SysInfo, $SysCInfo) = readSysInfo($OStarget);
     
     foreach (keys(%{$SysCInfo->{"non_prefix"}}))
     {
@@ -1466,7 +1473,7 @@ sub dumpSystem($)
             if(not defined $LibSoname{$LName}) {
                 $LibSoname{$LName}=$Soname;
             }
-            if(-l $LPath and my $Path = resolve_symlink($LPath))
+            if(-l $LPath and my $Path = realpath($LPath))
             {
                 my $Name = get_filename($Path);
                 if(not defined $LibSoname{$Name}) {
@@ -1572,7 +1579,7 @@ sub dumpSystem($)
         }
         if(-l $LPath)
         { # symlinks
-            if(my $Path = resolve_symlink($LPath)) {
+            if(my $Path = realpath($LPath)) {
                 $SysLibs{$Path} = 1;
             }
         }
@@ -1586,7 +1593,7 @@ sub dumpSystem($)
                 {
                     my $Candidate = $Candidates[0];
                     if(-l $Candidate
-                    and my $Path = resolve_symlink($Candidate)) {
+                    and my $Path = realpath($Candidate)) {
                         $Candidate = $Path;
                     }
                     $SysLibs{$Candidate} = 1;
@@ -1728,7 +1735,7 @@ sub dumpSystem($)
         my $Count = keys(%{$SysLib_Symbols{$LPath}});
         my %Prefixes = %{$LibPrefix{$LPath}};
         my @Prefixes = sort {$Prefixes{$b}<=>$Prefixes{$a}} keys(%Prefixes);
-        # print "$LPath ".Dumper(\%Prefixes);
+        
         if($#Prefixes>=1)
         {
             my $MaxPrefix = $Prefixes[0];
@@ -1763,7 +1770,6 @@ sub dumpSystem($)
                     next if($MaxPrefix=~/\Q$Prefix\E/i);
                     next if($Prefix=~/\Q$SName\E/i);
                     
-                    # print "Removing $Prefix $Num\n";
                     foreach my $Symbol (keys(%{$PrefixSymbols{$LPath}{$Prefix}})) {
                         delete($SysLib_Symbols{$LPath}{$Symbol});
                     }
